// CPlusPlusSocketCommunicationWithoutVISA.cpp : Defines the entry point for the console application.
//

// this example works using Microsoft Visual Studio 2012
// Be sure to include the following library when linking:
// WS2_32.lib 
// Project->Poperties->Linker->Input->additional Dependencies
// Set IP address with inet_addr function
// Set port with htons function
// 8800 Communications Setup should have 'Last Input' for terminator and enable 'Send Acknowledgement'
// This example uses 192.168.4.97 for IP and 8888 for port

#include "stdafx.h"
#include "iostream"
#include "atlstr.h"
using namespace std;

int _tmain(int argc, _TCHAR* argv[])
{
  char commandArray[256];

  //Initialize Windows Sockets
  WSADATA wsaData;
  WSAStartup(MAKEWORD(2,2), &wsaData);
  //Remote Socket
  sockaddr_in saRemote;
  memset(&saRemote, 0, sizeof(saRemote));
  //Setup remote Chamber socket info (IP address and TCP Port)
  saRemote.sin_family = AF_INET;
  saRemote.sin_addr.s_addr = inet_addr("192.168.4.97");
  //Remote Chamber Controller IP Address
  saRemote.sin_port = htons(8888);
  //Remote Chamber Controller TCP Port
  //Create a new TCP Socket
  SOCKET s = socket(AF_INET, SOCK_STREAM, 0);
  //Connect to remote Chamber Controller
  connect(s, (const sockaddr *)&saRemote, sizeof(saRemote));
  //IDEN? command will request identification string from Chamber Controller
  // May use *idn? also
  string command = "IDEN?\r";
  //Note we are using \r as termination character
  //Send command to Chamber Controller
  send(s, command.c_str(), command.length(), 0);
  //Temp buffer used to store incoming data from Chamber Controller
  char buffer[128];
  memset(buffer, 0, sizeof(buffer));
  //Receive Chamber response
  recv(s, buffer, sizeof(buffer), 0);
  cout << "Chamber Controller Identified as: " << buffer << endl;
  Sleep(1000);
  command = "*idn?\r";
  send(s, command.c_str(), command.length(), 0);
  memset(buffer, 0, sizeof(buffer));
  recv(s, buffer, sizeof(buffer), 0);
  cout << "Chamber Controller Identified as: " << buffer << endl;
  Sleep(1000);

  // 'RUNM\r' command will start the chamber in manual mode
  command = "RUNM\r";
  send(s, command.c_str(), command.length(), 0);
  memset(buffer, 0, sizeof(buffer));
  recv(s, buffer, sizeof(buffer), 0);

  for (int nSetpt = 0; nSetpt < 10; nSetpt++)  // loop through 10 setpoints 25 to 34 deg
  {
    Sleep(1000);
    // 'SETP1,25\r' command will set the current manual mode setpoint for channel 1 process variable
    sprintf_s(commandArray, sizeof(commandArray), "SETP1,%d\r",(nSetpt + 25));
    //Note we are using \r as termination character
    //Send command to Chamber Controller
    send(s, commandArray, strlen(commandArray), 0);
    memset(buffer, 0, sizeof(buffer));
    //Receive Chamber response.  Note that for write only commands that do not expect a response, the chamber controller returns the status byte
    // You MUST read the response even after write only commands!
    recv(s, buffer, sizeof(buffer), 0);

    //PVAR1? requests the Process Variable for Channel 1
	  //Refer to Thermotron Chamber Controller Manual for rest of I/O commands for your model
	  command = "PVAR1?\r";
	  //Send the Query Command
	  send(s, command.c_str(), command.length(), 0);
	  //Clean the receive buffer
	  memset(buffer, 0, sizeof(buffer));
	  //Receive response
	  recv(s, buffer, sizeof(buffer), 0);
	  //Print the temperature to console
	  cout << "Current Chamber Temperature: " << buffer << endl;

	  //SETP1? requests the Setpoint Variable for Channel 1
	  command = "SETP1?\r";
	  send(s, command.c_str(), command.length(), 0);
	  memset(buffer, 0, sizeof(buffer));
	  recv(s, buffer, sizeof(buffer), 0);
	  cout << "Current Chamber Setpoint: " << buffer << endl;
  }

  // 'STOP\r' command will stop the chamber
  command = "STOP\r";
  send(s, command.c_str(), command.length(), 0);
  memset(buffer, 0, sizeof(buffer));
  recv(s, buffer, sizeof(buffer), 0);

  //Close socket connection
  closesocket(s);
  //Shut down WinSock
  WSACleanup();

  printf("Press <Enter> to continue");
  char chReturnValue = getchar();

	return 0;
}

