// C++ tcp socket chamber io using visa.cpp : Defines the entry point for the console application.

#include "stdafx.h"
#include <windows.h>
#include "visa.h"

// chamber controller Communication settings for tcp:
// Terminator:  Last Input
// Send Acknowledgement: checked
// Command Compatibility: 8800
// Be sure to include visa32.lib, visa.h, in project
// Add include directory:  Project -> Configuration Properties -> C/C++ -> General -> Additional Include Directories
// (c:\Program Files (x86)\IVI Foundation\VISA\WinNT\include)

// Project  Configuration Properties  Linker  General  Additional Library Directories
// c:\Program Files (x86)\IVI Foundation\VISA\WinNT\lib\msc

#define READ_BUFFER_SIZE 4096
static unsigned char data[READ_BUFFER_SIZE];
static ViStatus status;
static ViSession inst;
static ViUInt32 BytesToWrite;
static ViSession defaultRM;
static ViUInt32 rcount;
static char stringBuffer[256];
static char stringinput[256];

int _tmain(int argc, _TCHAR* argv[])
{

   /*
    *  First we open a session to the VISA resource manager.  We are
    *  returned a handle to the resource manager session that we must
    *  use to open sessions to specific instruments.
    */
   status = viOpenDefaultRM (&defaultRM);
   if (status < VI_SUCCESS)
   {
      printf("Could not open a session to the VISA Resource Manager!\n");
      exit (EXIT_FAILURE);
   }
   
   /*
    *  Next we use the resource manager handle to open a session to a
    *  tcp instrument at ip xxx.xxx.xxx.xxx.  A handle to this session is
    *  returned in the handle inst.  Please consult the NI-VISA User Manual (http://www.ni.com/pdf/manuals/370423a.pdf)
    *  for the syntax for using other instruments.
    */
   // status = viOpen (defaultRM, "TCPIP::xxx.xxx.xxx.xxx::xxxx::SOCKET", VI_NULL, VI_NULL, &inst);
   // For IP 192.168.4.97 port 8888 use:
   status = viOpen (defaultRM, "TCPIP::192.168.4.97::8888::SOCKET", VI_NULL, VI_NULL, &inst);

   // set socket timeout to 5000 mS
   status = viSetAttribute(inst, VI_ATTR_TMO_VALUE, 5000);
   // NOTE:::  Use Last Input for terminator on chamber controller
   status = viSetAttribute(inst, VI_ATTR_TERMCHAR, 13);// set termination char to CR (defaults to LF \n)
   //(VI_ATTR_TERMCHAR_EN defaults to false so this needs to be set to true)
   status = viSetAttribute(inst, VI_ATTR_TERMCHAR_EN, 1);// enable string termination with char instead of timeout

   memset(data, 0, sizeof(data));  // init data buffer to all zeros
   printf("Waiting a second for socket initialization...\n");
   Sleep(1000L);
   
   printf("write 'IDEN?' to chamber controller to get Identification\n");
   strcpy_s(stringinput,"IDEN?\r"); // be sure to use CR for message terminator
   BytesToWrite = (ViUInt32)strlen(stringinput);
   status = viWrite(inst, (ViBuf)stringinput, BytesToWrite, &rcount);

   printf("begin read\n");
   status = viRead(inst, data, READ_BUFFER_SIZE - 1, &rcount);
   printf ("Here is the data:  %s\n\n", data);

  printf("write 'RUNM' to chamber controller to start running the chamber\n");
  strcpy_s(stringinput,"RUNM\r"); // be sure to use CR for message terminator
  BytesToWrite = (ViUInt32)strlen(stringinput);
  status = viWrite(inst, (ViBuf)stringinput, BytesToWrite, &rcount);

  printf("begin read\n"); // Always read after a write.  If not requesting data from a write, then error status is returned.
  memset(data, 0, sizeof(data));  // init data buffer to all zeros
  status = viRead(inst, data, READ_BUFFER_SIZE - 1, &rcount);
  printf ("data:  %s\n\n", data);

   for (int nLoop = 0; nLoop  < 10; nLoop ++)
   {
     sprintf_s(stringBuffer,"SETP1,%d\r",(25 + nLoop));
     BytesToWrite = (ViUInt32)strlen(stringBuffer);
     status = viWrite(inst, (ViBuf)stringBuffer, BytesToWrite, &rcount);
     int nLength = strlen(stringBuffer);
     stringBuffer[nLength - 1] = 0; // remove CR terminator from string
     printf("write '%s' to chamber controller to set current channel 1 setpoint\n", stringBuffer);
     memset(data, 0, sizeof(data));  // init data buffer to all zeros
     status = viRead(inst, data, READ_BUFFER_SIZE - 1, &rcount);
     printf ("data:  %s\n\n", data);

     printf("write 'PVAR1' to chamber controller to get Process Variable channel 1 value\n");
     strcpy_s(stringinput,"PVAR1?\r"); // be sure to use CR for message terminator
     BytesToWrite = (ViUInt32)strlen(stringinput);
     status = viWrite(inst, (ViBuf)stringinput, BytesToWrite, &rcount);
     memset(data, 0, sizeof(data));  // init data buffer to all zeros
     status = viRead(inst, data, READ_BUFFER_SIZE - 1, &rcount);
     printf ("data:  %s\n\n", data);

     printf("write 'SETP1?' to chamber controller to get current channel 1 setpoint\n");
     strcpy_s(stringinput,"SETP1?\r"); // be sure to use CR for message terminator
     BytesToWrite = (ViUInt32)strlen(stringinput);
     status = viWrite(inst, (ViBuf)stringinput, BytesToWrite, &rcount);
     memset(data, 0, sizeof(data));  // init data buffer to all zeros
     status = viRead(inst, data, READ_BUFFER_SIZE - 1, &rcount);
     printf ("data:  %s\n\n", data);

     Sleep(1000);
   } // end of loop

  printf("write 'STOP' to chamber controller to stop the running chamber\n");
  strcpy_s(stringinput,"STOP\r"); // be sure to use CR for message terminator
  BytesToWrite = (ViUInt32)strlen(stringinput);
  status = viWrite(inst, (ViBuf)stringinput, BytesToWrite, &rcount);
  memset(data, 0, sizeof(data));  // init data buffer to all zeros
  status = viRead(inst, data, READ_BUFFER_SIZE - 1, &rcount);
  printf ("data:  %s\n\n", data);

  printf("\nHit enter to continue.");
  getchar();

  /*
  *  Now we close the instrument session and the resource manager
  *  session to free up resources.
  */
  status = viClose(inst);
  status = viClose(defaultRM);

	return 0;
}

